<?php
if ( !function_exists( "add_action" ) ) {
	echo "Hi there!  I'm just a plugin, not much I can do when called directly.";
	exit;
}


class Gifthuntfree {
  private $gifthunt_sessions_table;
  private $gifthunt_gifts_table;
  private $gifthunt_hunters_table;

  private $db;
  private $prefix;

  public $id = 0;
  public $name;
  public $active = false;
  public $visible_from;
  public $visible_to;
  public $icon = 1;
  public $custom_icon = "";
  public $icon_placement = "random";
  public $icon_animation = "pop";
  public $display_type = "time";
  public $time_to_display = 90;
  public $pageview_to_display = 5;
  public $allow_multiple_collect = false;
  public $gift_type = "oneMany";
  public $gifts;
  public $popup_title = "You just found a gift";
  public $popup_body = "This is your lucky day because you just found a hidden gift on our website. Fill the form below to claim your prize.";
  public $popup_button = "Submit";
  public $popup_submit_body = "Thank you for filling the form. We've sent you an email with your gift and you can also find it below.";
  public $popup_design = "default";
  public $form_collect_first_name = true;
  public $form_collect_last_name = true;
  public $form_collect_additional_information = false;
  public $form_label_first_name = "First name";
  public $form_label_last_name = "Last name";
  public $form_label_email = "Email address";
  public $form_label_additional_information = "Additional information";
  public $form_label_legal = "I have read and agree to the Privacy Policy and Terms of Service";
  public $form_legal_url;
  public $success_email_send = 1;
  public $success_email_sender_name;
  public $success_email_sender_email;
  public $success_email_subject = "🎁 Here comes your gift";
  public $success_email_body = "Hi %firstname%,\n\nthe gift you just found on our website is the following:\n%gift%\n\nMake sure you keep this email in a good place so you'll find the gift when you need it. If you have any questions, don't hesitate to get in touch. For contact information, visit our website.\n\nHave a great day.";
  public $created;
  public $updated;

  public function __construct() {
    global $wpdb;

    $this->db = $wpdb;
    $this->prefix = $wpdb->prefix;

    $this->gifthunt_sessions_table = $wpdb->prefix . "ff_gifthuntfree_sessions";
    $this->gifthunt_gifts_table = $wpdb->prefix . "ff_gifthuntfree_gifts";
    $this->gifthunt_hunters_table = $wpdb->prefix . "ff_gifthuntfree_hunters";

    $date = new DateTime("now");
    $this->name = "Gift Hunt Session " . $date->format( "Y-m-d" );

    $this->visible_from = $date->format( "Y-m-d" );

    $date->add( new DateInterval("P30D") );
    $this->visible_to = $date->format( "Y-m-d" );

    $this->success_email_sender_name = get_bloginfo( "name" );
    $this->success_email_sender_email = get_option( "admin_email" );

    $this->form_legal_url = get_option( "url" );
  }

  /**
   * Create necessary data tables after Gifthunt installed
   */
  public function install_gifthunt() {
    if ( !current_user_can( "manage_options" ) ) {
      wp_die( "Unauthorized user" );
    }

    $sessions_table_name = $this->gifthunt_sessions_table;
    $gifts_table_name = $this->gifthunt_gifts_table;
    $hunters_table_name = $this->gifthunt_hunters_table;

    if ( !function_exists( "dbDelta" ) ) { 
      require_once ABSPATH . "/wp-admin/includes/upgrade.php"; 
    }

    /**
     * Create sessions table
     */
    $sessions_table_create_query = "CREATE TABLE " . $sessions_table_name . " ( 
      id int(11) NOT NULL auto_increment ,
      name varchar(250) NOT NULL ,
      active boolean NOT NULL ,
      visible_from date NOT NULL ,
      visible_to date NOT NULL ,
      icon varchar(250) NOT NULL ,
      custom_icon text ,
      icon_placement varchar(20) NOT NULL,
      icon_animation varchar(20) NOT NULL,
      display_type varchar(10) NOT NULL ,
      time_to_display int(5) NOT NULL ,
      pageview_to_display int(5) NOT NULL ,
      allow_multiple_collect boolean NOT NULL ,
      gift_type varchar(10) NOT NULL ,
      popup_title varchar(250) NOT NULL ,
      popup_body text NOT NULL ,
      popup_button varchar(100) NOT NULL ,
      popup_submit_body text NOT NULL ,
      popup_design varchar(20) NOT NULL,
      form_collect_first_name boolean NOT NULL ,
      form_collect_last_name boolean NOT NULL ,
      form_collect_additional_information boolean NOT NULL ,
      form_label_first_name varchar(100) NOT NULL ,
      form_label_last_name varchar(100) NOT NULL ,
      form_label_email varchar(100) NOT NULL ,
      form_label_additional_information varchar(100) NOT NULL ,
      form_label_legal varchar(250) NOT NULL ,
      form_legal_url varchar(250) NOT NULL ,
      success_email_send boolean NOT NULL DEFAULT 1 ,
      success_email_sender_name varchar(100) NOT NULL ,
      success_email_sender_email varchar(100) NOT NULL ,
      success_email_subject varchar(150) NOT NULL ,
      success_email_body text NOT NULL ,
      created timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ,
      updated timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ,
      PRIMARY KEY  (id)
      )
      CHARSET=utf8mb4
      COLLATE utf8mb4_unicode_ci";

    dbDelta( $sessions_table_create_query );

    /**
     * Create gifts table
     */
    $gifts_table_create_query = "CREATE TABLE " . $gifts_table_name . " (
      id int(11) NOT NULL auto_increment,
      session_id int(11) NOT NULL,
      gift text NOT NULL,
      status varchar(10) NOT NULL,
      created timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
      updated timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
      PRIMARY KEY  (id)
      )
      CHARSET=utf8mb4
      COLLATE utf8mb4_unicode_ci";

    dbDelta( $gifts_table_create_query );

    /**
     * Create hunters table
     */
    $hunters_table_create_query = "CREATE TABLE " . $hunters_table_name . " (
      id int(11) NOT NULL auto_increment , 
      session_id int(11) NOT NULL , 
      first_name varchar(100) NOT NULL , 
      last_name varchar(100) NOT NULL , 
      email varchar(100) NOT NULL , 
      additional_information text NOT NULL , 
      gift text NOT NULL , 
      gift_id int(11) NOT NULL , 
      created timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP , 
      PRIMARY KEY  (id)
      )
      CHARSET=utf8mb4
      COLLATE utf8mb4_unicode_ci";

    dbDelta( $hunters_table_create_query );
  }

  /**
   * Remove created database tables on uninstall
   */
  public function uninstall_gifthunt() {
    if (!current_user_can("manage_options")) {
      wp_die("Unauthorized user");
    }

    // remove gifts table
    $this->db->query( "DROP TABLE IF EXISTS `" . $this->gifthunt_gifts_table . "`" );

    // remove hunters table
    $this->db->query( "DROP TABLE IF EXISTS `" . $this->gifthunt_hunters_table . "`" );

    // remove sessions table
    $this->db->query( "DROP TABLE IF EXISTS `" . $this->gifthunt_sessions_table . "`" );

    return;
  }

  /**
   * Get all gifthunt sessions from the database
   */
  public function get_gifthunt_list() {
    $gifthunt_sessions = $this->db->get_results(
      "
      SELECT *
      FROM " . $this->gifthunt_sessions_table . "
      ORDER BY id DESC
      "
    );

    return $gifthunt_sessions;
  }
  
  /**
   * Get the data of a selected gifthunt session from the database
   */
  public function get_gifthunt_data( $gifthunt_id ) {
    $current_gifthunt_id = intval( $gifthunt_id );
    $gifthunt_data = $this->db->get_row(
      "
      SELECT *
      FROM " . $this->gifthunt_sessions_table . "
      WHERE id = " . $current_gifthunt_id . "
      "
    );

    if ( $gifthunt_data == null ) {
      return false;
    }

    // get the gifts from the database for selected session
    $gifts = $this->db->get_results(
      "
      SELECT *
      FROM " . $this->gifthunt_gifts_table . "
      WHERE session_id = " . $current_gifthunt_id . "
      AND status != 'deleted'
      ORDER BY id ASC
      "
    );

    $gifthunt_data->{ "gifts" } = $gifts;

    return stripslashes_deep( $gifthunt_data );
  }

  /**
   * Process and check posted gifthunt session data
   */
  public function process_gifthunt_session_data() {
    $result = [
      "status"  => "error",
      "data"    => ""
    ];

    $error_count = 0;
    // check required fields
    $required_basic_fields = [ "name", "popup_title", "form_label_email", "form_label_legal", "form_legal_url", "popup_button", "success_email_sender_name", "success_email_sender_email", "success_email_subject", "popup_body", "popup_submit_body", "success_email_body", "icon", "display_type", "gift_type" ];

    // check display type
    if ( $_POST["gifthuntSessionData"]["display_type"] == "time" ) {
      array_push( $required_basic_fields, "time_to_display" );
    } else {
      array_push( $required_basic_fields, "pageview_to_display" );
    }

    // check gift type
    if ( $_POST["gifthuntSessionData"]["gift_type"] == "oneMany" ) {
      array_push( $required_basic_fields, "gift" );
    } else {
      if ( ! $_POST["gifthuntSessionData"]["gifts"][0]["value"] ) {
        $error_count++;
      }
    }

    // check custom icon
    if( $_POST["gifthuntSessionData"]["icon"] == 99 && empty( $_POST["gifthuntSessionData"]["custom_icon"] )){
      $error_count++;
    }

    foreach ( $required_basic_fields as $required_basic_field ) {
      if ( ! $_POST["gifthuntSessionData"][ $required_basic_field ] || strlen( $_POST["gifthuntSessionData"][ $required_basic_field ] ) < 1 ) {
        $error_count++;
      }
    }

    if ( $_POST["gifthuntSessionData"]["form_collect_first_name"] == "true" && ( ! $_POST["gifthuntSessionData"]["form_label_first_name"] || strlen( $_POST["gifthuntSessionData"]["form_label_first_name"] ) < 1 ) ) {
      $error_count++;
    }

    if ( $_POST["gifthuntSessionData"]["form_collect_last_name"] == "true" && ( ! $_POST["gifthuntSessionData"]["form_label_last_name"] || strlen( $_POST["gifthuntSessionData"]["form_label_last_name"] ) < 1 ) ) {
      $error_count++;
    }

    if ( $_POST["gifthuntSessionData"]["form_collect_additional_information"] == "true" && ( ! $_POST["gifthuntSessionData"]["form_label_additional_information"] || strlen( $_POST["gifthuntSessionData"]["form_label_additional_information"] ) < 1 ) ) {
      $error_count++;
    }

    if ( $_POST["gifthuntSessionData"]["active"] == "true" && ( ! $_POST["gifthuntSessionData"]["visible_from"] || !$_POST["gifthuntSessionData"]["visible_to"] ) ) {
      $error_count++;
    }

    if ($error_count) {
      $result["data"] = "Please, fill all required fields to create a new Gift Hunt session";
      return $result;
    }

    // process logical session data
    $gifthunt_session_visible_from = isset( $_POST["gifthuntSessionData"]["visible_from"] ) ? sanitize_text_field( $_POST["gifthuntSessionData"]["visible_from"] ) : $this->visible_from;
    $gifthunt_session_visible_to = isset( $_POST["gifthuntSessionData"]["visible_to"] ) ? sanitize_text_field( $_POST["gifthuntSessionData"]["visible_to"] ) : $this->visible_to;
    $gifthunt_time_to_display = isset( $_POST["gifthuntSessionData"]["time_to_display"] ) ? intval( $_POST["gifthuntSessionData"]["time_to_display"] ) : $this->time_to_display;
    $gifthunt_pageview_to_display = isset( $_POST["gifthuntSessionData"]["pageview_to_display"] ) ? intval( $_POST["gifthuntSessionData"]["pageview_to_display"] ) : $this->pageview_to_display;
    $gifthunt_form_label_first_name = isset( $_POST["gifthuntSessionData"]["form_label_first_name"] ) ? sanitize_text_field( $_POST["gifthuntSessionData"]["form_label_first_name"] ) : $this->form_label_first_name;
    $gifthunt_form_label_last_name = isset( $_POST["gifthuntSessionData"]["form_label_last_name"] ) ? sanitize_text_field( $_POST["gifthuntSessionData"]["form_label_last_name"] ) : $this->form_label_last_name;
    $gifthunt_form_label_additional_information = isset( $_POST["gifthuntSessionData"]["form_label_additional_information"] ) ? sanitize_text_field( $_POST["gifthuntSessionData"]["form_label_additional_information"] ) : $this->form_label_additional_information;

    // add gifts to the gifthunt session object
    $gifthunt_session_gifts = array();
    if ( $_POST["gifthuntSessionData"]["gift_type"] == "oneMany" ) {
      array_push( $gifthunt_session_gifts, [
        "dbid" => 0, 
        "value" => sanitize_text_field( $_POST["gifthuntSessionData"]["gift"] )
        ]);
    } else {
      if ( is_array( $_POST["gifthuntSessionData"]["gifts"] ) ) {
        foreach( $_POST["gifthuntSessionData"]["gifts"] as $gift){
          array_push( $gifthunt_session_gifts, [
            "dbid" => intval( $gift["dbid"] ), 
            "value" => sanitize_text_field( $gift["value"] )
            ]);
        }
      }
    }

    $gifthunt_session_db_data = [
      "name"                                => sanitize_text_field( $_POST["gifthuntSessionData"]["name"] ),
      "active"                              => intval( $_POST["gifthuntSessionData"]["active"] ),
      "visible_from"                        => $gifthunt_session_visible_from,
      "visible_to"                          => $gifthunt_session_visible_to,
      "icon"                                => intval( $_POST["gifthuntSessionData"]["icon"] ),
      "custom_icon"                         => urlencode( $_POST["gifthuntSessionData"]["custom_icon"] ),
      "display_type"                        => sanitize_text_field( $_POST["gifthuntSessionData"]["display_type"] ),
      "time_to_display"                     => $gifthunt_time_to_display,
      "pageview_to_display"                 => $gifthunt_pageview_to_display,
      "allow_multiple_collect"              => intval( $_POST["gifthuntSessionData"]["allow_multiple_collect"] ),
      "gift_type"                           => sanitize_text_field( $_POST["gifthuntSessionData"]["gift_type"] ),
      "popup_title"                         => sanitize_text_field( $_POST["gifthuntSessionData"]["popup_title"] ),
      "popup_body"                          => wp_kses_post( $_POST["gifthuntSessionData"]["popup_body"] ),
      "popup_button"                        => sanitize_text_field( $_POST["gifthuntSessionData"]["popup_button"] ),
      "popup_submit_body"                   => wp_kses_post( $_POST["gifthuntSessionData"]["popup_submit_body"] ),
      "form_collect_first_name"             => intval( $_POST["gifthuntSessionData"]["form_collect_first_name"] ),
      "form_collect_last_name"              => intval( $_POST["gifthuntSessionData"]["form_collect_last_name"] ),
      "form_collect_additional_information" => intval( $_POST["gifthuntSessionData"]["form_collect_additional_information"] ),
      "form_label_first_name"               => $gifthunt_form_label_first_name,
      "form_label_last_name"                => $gifthunt_form_label_last_name,
      "form_label_email"                    => sanitize_text_field( $_POST["gifthuntSessionData"]["form_label_email"] ),
      "form_label_additional_information"   => sanitize_text_field( $gifthunt_form_label_additional_information ),
      "form_label_legal"                    => sanitize_text_field( $_POST["gifthuntSessionData"]["form_label_legal"] ),
      "form_legal_url"                      => esc_url( $_POST["gifthuntSessionData"]["form_legal_url"] ),
      "success_email_send"                  => intval( $_POST["gifthuntSessionData"]["success_email_send"] ),
      "success_email_sender_name"           => sanitize_text_field( $_POST["gifthuntSessionData"]["success_email_sender_name"] ),
      "success_email_sender_email"          => sanitize_email( $_POST["gifthuntSessionData"]["success_email_sender_email"] ),
      "success_email_subject"               => sanitize_text_field( $_POST["gifthuntSessionData"]["success_email_subject"] ),
      "success_email_body"                  => wp_kses_post( $_POST["gifthuntSessionData"]["success_email_body"] ),
      "created"                             => date("Y-m-d H:i:s"),
      "updated"                             => date("Y-m-d H:i:s"),
      "gifts"                               => $gifthunt_session_gifts
    ];

    if ( isset( $_POST["gifthuntSessionData"]["id"] ) && $_POST["gifthuntSessionData"]["id"] > 0 ) {
      $gifthunt_session_db_data["id"] = intval( $_POST["gifthuntSessionData"]["id"] );
    }

    return $gifthunt_session_db_data;
  }

  /**
   * Save data of a new Gifthunt session
   */
  public function create_gifthunt() {
    $result = [
      "status"  => "error",
      "data"    => ""
    ];

    $gifthunt_session_db_data = $this->process_gifthunt_session_data();

    if ( isset( $gifthunt_session_db_data["status"] ) ) {
      return $gifthunt_session_db_data;
    }

    $gifthunt_session_gifts = $gifthunt_session_db_data["gifts"];

    unset( $gifthunt_session_db_data["gifts"] );

    // check if current gifthunt session is active
    // turn off other active session
    if ( $gifthunt_session_db_data["active"] ) {
      $this->db->update( $this->gifthunt_sessions_table, array( "active" => 0 ), array( "active" => 1 ) );
    }

    $insert_result = $this->db->insert(
      $this->gifthunt_sessions_table,
      $gifthunt_session_db_data
    );

    $new_session_id = $this->db->insert_id;

    if ( ! $new_session_id ) {
      $result["data"] = "Database error. Please, wait a few minutes and try again";
      return $result;
    }

    // save gifts to the database
    $gifts = array();

    foreach ( $gifthunt_session_gifts as $gift ) {
      $gifts[] = $this->db->prepare( "( %d, %s, %s)" , $new_session_id, sanitize_text_field( $gift["value"] ), "new");
    }

    $gifts_query = "INSERT INTO `" . $this->gifthunt_gifts_table . "` (`session_id`, `gift`, `status`) VALUES ";
    $gifts_query .= implode( ", ", $gifts );

    $gifts_query_result = $this->db->query( $gifts_query );
    
    if ( ! $gifts_query_result ) {
      $result["data"] = "Database error. Please, wait a few minutes and try again";
      return $result;
    }

    // return new session id
    $result["status"] = "success";
    $result["data"] = $new_session_id;

    return $result;
  }

  /**
   * Update data of a selected Gifthunt session
   */
  public function update_gifthunt() {
    $result = [
      "status"  => "error",
      "data"    => ""
    ];

    $gifthunt_session_db_data = $this->process_gifthunt_session_data();

    if ( isset( $gifthunt_session_db_data["status"] ) ) {
      return $gifthunt_session_db_data;
    }

    $gifthunt_session_gifts = $gifthunt_session_db_data["gifts"];
    $gifthunt_session_id = $gifthunt_session_db_data["id"];

    unset( $gifthunt_session_db_data["gifts"] );
    unset( $gifthunt_session_db_data["id"] );

    // check if current gifthunt session is active
    // turn off other active session
    if ( $gifthunt_session_db_data["active"] ) {
      $this->db->update( $this->gifthunt_sessions_table, array( "active" => 0 ), array( "active" => 1 ) );
    }

    // update session default data
    $update_result = $this->db->update(
      $this->gifthunt_sessions_table,
      $gifthunt_session_db_data,
      array( "id" => $gifthunt_session_id )
    );

    if ( $update_result === false ) {
      $result["data"] = "Database error. Please, wait a few minutes and try again";
      return $result;
    }

    // update gifts and save new gifts
    $gifts = array();
    $gift_id_array = array();

    foreach ( $gifthunt_session_gifts as $gift ) {
      $gifts[] = $this->db->prepare( "( %d, %d, %s, %s)" , $gift["dbid"], $gifthunt_session_id, sanitize_text_field( $gift["value"] ), "new" );
      if ( $gift["dbid"] != 0 ) {
        $gift_id_array[] = $gift["dbid"];
      }
    }

    if ( ! count( $gift_id_array ) ) {
      $gift_id_array[0] = 0;
    }

    // set gifts to deleted that are not in the gift ids array
    $gifts_delete_query = "UPDATE " . $this->gifthunt_gifts_table . " SET status = 'deleted' WHERE session_id = " . $gifthunt_session_id . " AND id NOT IN ( ";
    $gifts_delete_query .= implode( ", ", $gift_id_array );
    $gifts_delete_query .= " );";
    $this->db->query( $gifts_delete_query );

    // insert new gifts to the database and update old ones
    $gifts_query = "INSERT INTO `" . $this->gifthunt_gifts_table . "` (`id`, `session_id`, `gift`, `status`) VALUES ";
    $gifts_query .= implode( ", ", $gifts );
    $gifts_query .= " ON DUPLICATE KEY UPDATE gift = VALUES(gift);";

    $gifts_query_result = $this->db->query( $gifts_query );

    if ( $gifts_query_result === false ) {
      $result["data"] = $this->db->print_error() . " Database error. Please, wait a few minutes and try again";
      return $result;
    }

    // get valid gifts from the database
    $session_gifts = $this->db->get_results(
      "
      SELECT *
      FROM " . $this->gifthunt_gifts_table . "
      WHERE session_id = " . $gifthunt_session_id . "
      AND status != 'deleted'
      ORDER BY id ASC
      "
    );

    // return success message
    $result["status"] = "success";
    $result["data"] = "Session updated successfully";
    $result["gifts"] = stripslashes_deep( $session_gifts );

    return $result;
  }

  /**
   * Delete a Gifthunt session
   */
  public function delete_gifthunt() {
    $gifhunt_id = intval( $_POST["id"] );
    $result = [
      "status"  => "error",
      "data"    => ""
    ];

    // delete hunters
    $hunters_delete_result = $this->db->delete( $this->gifthunt_hunters_table, array( "session_id" => $gifhunt_id ), array( '%d' ) );

    // delete gifts
    $gifts_delete_result = $this->db->delete( $this->gifthunt_gifts_table, array( "session_id" => $gifhunt_id ), array( '%d' ) );

    // delete session
    $session_delete_result = $this->db->delete( $this->gifthunt_sessions_table, array( "id" => $gifhunt_id ), array( '%d' ) );

    if (!$session_delete_result || !$gifts_delete_result) {
      $result["data"] = "There was an error during the process. Please, reload the page and try again";
    } else {
      $result["status"] = "success";
    }

    return $result;
  }

  /**
   * Delete hunter data
   */
  public function delete_hunter() {
    $current_hunter_id = intval( $_POST["id"] );
    $result = [
      "status"  => "error",
      "data"    => ""
    ];

    // get hunter data from the database
    $hunter_data = $this->db->get_row(
      "
      SELECT *
      FROM " . $this->gifthunt_hunters_table . "
      WHERE id = " . $current_hunter_id . "
      "
    );

    if (!$hunter_data) {
      $result["data"] = "There was an error during the process. Please, try again";
      return $result;
    }

    $hunter_delete_result = $this->db->delete( $this->gifthunt_hunters_table, array( "id" => $current_hunter_id ), array( '%d' ) );

    if (!$hunter_delete_result) {
      $result["data"] = "There was an error during the process. Please, try again";
      return $result;
    }

    // change gift status to new
    $gift_update_result = $this->db->update(
      $this->gifthunt_gifts_table,
      array(
        "status"  => "new"
      ),
      array(
        "id"      => $hunter_data->gift_id,
        "status"  => "found"
      ),
      array( '%s' ),
      array( '%d' )
    );

    $result["status"] = "success";
    $result["data"] = "Hunter has been removed";

    return $result;
  }

  /**
   * Get all hunters by gifthunt id
   */
  public function get_hunters( $gifthunt_id ) {
    $current_gifthunt_id = intval( $gifthunt_id );

    $session_hunters = $this->db->get_results(
      "
      SELECT *
      FROM " . $this->gifthunt_hunters_table . "
      WHERE session_id = " . $current_gifthunt_id . "
      ORDER BY created DESC
      "
    );

    return $session_hunters;
  }

  /**
   * Delete all hunters
   */
  public function delete_hunters_all() {
    $current_gifthunt_id = intval( $_POST['id'] );
    $result = [
      "status"  => "error",
      "data"    => ""
    ];
    // delete hunters
    $hunters_delete_result = $this->db->delete( $this->gifthunt_hunters_table, array( "session_id" => $current_gifthunt_id ), array( '%d' ) );

    if ( ! $hunters_delete_result ) {
      $result["data"] = "There was an error during the process. Please, try again";
      return $result;
    }
    
    // update gifts status
    $gift_update_result = $this->db->update(
      $this->gifthunt_gifts_table,
      array(
        "status"      => "new"
      ),
      array(
        "session_id"  => $current_gifthunt_id,
        "status"      => "found"
      ),
      array( '%s' ),
      array( '%d' )
    );

    $result["status"] = "success";

    return $result;
  }

  /**
   * Get gifts for a specific gifthunt session
   */
  public function get_gifts( $gifthunt_id ) {
    $current_gifthunt_id = intval( $gifthunt_id );

    $gifts = $this->db->get_results(
      "
      SELECT *
      FROM " . $this->gifthunt_gifts_table . "
      WHERE session_id = " . $current_gifthunt_id . "
      ORDER BY id DESC
      "
    );

    return $gifts;
  }

  /**
   * Send test result mail
   */
  public function send_test_result_mail() {
    $result = [
      "status"  => "error",
      "data"    => ""
    ];

    $to = sanitize_email( $_POST["to"] );
    $from_name = sanitize_text_field( $_POST["success_email_sender_name"] );
    $from_email = sanitize_email( $_POST["success_email_sender_email"] );
    $subject = sanitize_text_field( "(TEST) " . $_POST["success_email_subject"] );
    $message = wp_kses_post( $_POST["success_email_body"] );

    $mail_result = wp_mail( $to, $subject, $message, "From: " . $from_name . " <" . $from_email . ">\r\nContent-Type: text/html; charset=UTF-8" );

    if ( ! $mail_result) {
      $result["data"] = "There was an error during the process, please wait a few minutes and try again";
    } else {
      $result["status"] = "success";
      $result["data"] = "A test mail has been sent to " . $to . "<br /> <strong>If you like it, don't forget to save your changes</strong>";
    }

    return $result;
  }

  /**
   * Export hunters
   */
  public function export_hunters() {
    $current_gifthunt_id = intval( $_POST["id"] );

    // get hunters from the database
    $hunters = $this->get_hunters( $current_gifthunt_id );

    if ( ! class_exists("XLSXWriter") ) {
      include "class-xlsxwriter.php";
    }
    
    $writer = new XLSXWriter();

    $filename = "gifhuntfree_hunters_" . date("YmdHis") . ".xlsx";
    
    header('Content-disposition: attachment; filename="'.XLSXWriter::sanitize_filename( $filename ).'"');
    header("Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
    header('Content-Transfer-Encoding: binary');
    header('Cache-Control: must-revalidate');
    header('Pragma: public');
    
    $writer->setAuthor( "Gifhunt for WordPress" );

    // sheet header
    $writer->writeSheetRow( "Hunters", array(
      "First name",
      "Last name",
      "Email",
      "Additional information",
      "Gift",
      "Date"
    ) );

    // sheet rows
    foreach ( $hunters as $hunter ) {
      $writer->writeSheetRow( "Hunters", array(
        stripslashes_deep( $hunter->first_name ),
        stripslashes_deep( $hunter->last_name ),
        stripslashes_deep( $hunter->email ),
        stripslashes_deep( $hunter->additional_information ),
        stripslashes_deep( $hunter->gift ),
        $hunter->created
      ) );
    }

    // display sheet in browser
    $writer->writeToStdOut();

    die();
  }

  /**
   * ====================
   * Frontend functions
   * ====================
   */
  private function get_active_gifthunt_data( $id = 0 ) {
    if ( $id != 0 ) {
      // get session data for preview
      $gifthunt_data = $this->db->get_row(
        "
        SELECT *
        FROM " . $this->gifthunt_sessions_table . "
        WHERE id = " . $id . "
        "
      );
    } else {
      $date = new DateTime("now");
      $today = $date->format( "Y-m-d" );
  
      $gifthunt_data = $this->db->get_row(
        "
        SELECT *
        FROM " . $this->gifthunt_sessions_table . "
        WHERE active = 1
        AND visible_from <= '" . $today . "'
        AND visible_to >= '" . $today . "'
        "
      );

      // check available gifts
      if ( $gifthunt_data && $gifthunt_data->gift_type == "moreOne" ) {
        $gifts_count = $this->db->get_var(
          "
          SELECT COUNT(*) 
          FROM " . $this->gifthunt_gifts_table . "
          WHERE status = 'new'
          AND session_id = " . $gifthunt_data->id . "
          "
        );

        if ( $gifts_count < 1 ) {
          return false;
        }
      }
    }

    if ( $gifthunt_data == null ) {
      return false;
    }

    return stripslashes_deep( $gifthunt_data );
  }

  /**
   * Display frontend script
   */
  public function display_frontend_script() {
    $main_css = file_get_contents( dirname( __DIR__ ) . "/assets/css/frontend/main.min.css" );
    $icon_css = file_get_contents( dirname( __DIR__ ) . "/assets/css/frontend/icon.min.css" );
    $main_script = file_get_contents( dirname( __DIR__ ) . "/assets/scripts/frontend/main.min.js" );
    $popup_window = file_get_contents( dirname( __DIR__ ) . "/assets/views/frontend/popup.html" );

    // get active session, or preview session data from the database
    // dont display anything if active session doesn't exists
    $session_preview = "false";
    if ( isset( $_GET["gfthntprvwfree"] ) ) {
      $session_preview = "true";
      $active_hunt_session_data = $this->get_active_gifthunt_data( intval( $_GET["gfthntprvwfree"] ) );
    } else {
      $active_hunt_session_data = $this->get_active_gifthunt_data();
    }

    if ( ! $active_hunt_session_data ) {
      echo "<!-- no active gifthunt session -->";
      return;
    }

    $gifthunt_frontend_script = "const ffDiscounthuntFreeSessionPreview = " . $session_preview . ";";
    $gifthunt_frontend_script .= "const ffDiscounthuntFreeNonce = '" . wp_create_nonce( "gifthuntfree-nonce-frontend" ) . "';";
    $gifthunt_frontend_script .= 'const ffDiscounthuntFreeCSS = "' . $main_css . '";';
    $gifthunt_frontend_script .= 'const ffDiscounthuntFreeIconCSS = "' . $icon_css . '";';
    $gifthunt_frontend_script .= "const ffDiscounthuntFreeSessionData = " . json_encode( $active_hunt_session_data ) . ";";
    $gifthunt_frontend_script .= "const ffDiscounthuntFreeApiEndpoint = '" . GIFTHUNTFREE_FRONTEND_ENDPOINT . "';";
    $gifthunt_frontend_script .= "const ffDiscounthuntFreeFrontendAssets = '" . GIFTHUNTFREE_FRONTEND_ASSETS_FOLDER . "';";
    $gifthunt_frontend_script .= "const ffDiscounthuntFreePopupWindow = " . $popup_window . ";";
    $gifthunt_frontend_script .= $main_script;

    echo "<script>" . $gifthunt_frontend_script . "</script>";
    
    return;
  }

  /**
   * Save hunter data, collect gift
   */
  public function collect_gift() {
    $result = [
      "status"  => "error",
      "data"    => ""
    ];

    $site_url = get_site_url();
    $referer = $_SERVER["HTTP_REFERER"];

    if ( strpos( $referer, $site_url ) === false ) {
      $result["status"] = "closed";
      return $result;
    }

    // check current session data
    $current_gifthunt_id = intval( $_POST["huntSession"] );
    $gifthunt_data = $this->db->get_row(
      "
      SELECT *
      FROM " . $this->gifthunt_sessions_table . "
      WHERE id = " . $current_gifthunt_id . "
      AND active = 1
      "
    );

    if ( $gifthunt_data == null ) {
      $result["status"] = "closed";
      return $result;
    }

    // get random gift from the dabatase
    $gifts = $this->db->get_results(
      "
      SELECT *
      FROM " . $this->gifthunt_gifts_table . "
      WHERE session_id = " . $current_gifthunt_id . "
      AND status = 'new'
      ORDER BY id DESC
      ",
      ARRAY_A
    );

    // out of gifts
    if ( count( $gifts ) < 1 ) {
      $result["status"] = "closed";
      return $result;
    }

    $collected_gift = $gifts[ mt_rand(0, count( $gifts ) - 1 ) ];

    // update gifts status if necessary
    if ( $gifthunt_data->gift_type == "moreOne" ) {
      $this->db->update(
        $this->gifthunt_gifts_table,
        array( "status" => "found" ),
        array( "id" => $collected_gift["id"] )
      );
    }

    $hunter_data = ["firstName", "lastName", "email", "additionalInformation"];
    foreach ( $hunter_data as $data ) {
      $hunter_data[ $data ] = sanitize_text_field( $_POST[ $data ] );
    }

    // save hunter data to the database
    $hunter_query_result = $this->db->query(
      $this->db->prepare(
        "
        INSERT INTO " . $this->gifthunt_hunters_table . "
        ( session_id, first_name, last_name, email, additional_information, gift, gift_id , created)
        VALUES ( %d, %s, %s, %s, %s, %s, %d, %s)
        ",
        $current_gifthunt_id,
        $hunter_data["firstName"],
        $hunter_data["lastName"],
        $hunter_data["email"],
        $hunter_data["additionalInformation"],
        $collected_gift["gift"],
        $collected_gift["id"],
        date( "Y-m-d H:i:s" )
      )
    );

    if ( ! $hunter_query_result ) {
      $result["data"] = "There was an error during the process. Please, wait a few minutes and try again";
      return $result;
    }

    if( $gifthunt_data->success_email_send == 1 ){
      // send mail to hunter
      $to = sanitize_email( $_POST["email"] );
      $from_name = stripslashes_deep( $gifthunt_data->success_email_sender_name );
      $from_email = stripslashes_deep( $gifthunt_data->success_email_sender_email );

      // replace placeholders with collected information
      $subject = str_replace( 
                  array( "%firstname%", "%lastname%", "%email%", "%gift%" ) , 
                  array( sanitize_text_field( $_POST["firstName"] ), sanitize_text_field( $_POST["lastName"] ), sanitize_email( $_POST["email"] ), stripslashes_deep( $collected_gift["gift"] ) ), 
                  stripslashes_deep( $gifthunt_data->success_email_subject )
                );

      $message = str_replace( 
                  array( "%firstname%", "%lastname%", "%email%", "%gift%" ) , 
                  array( $hunter_data["firstName"], $hunter_data["lastName"], $hunter_data["email"], stripslashes_deep( $collected_gift["gift"] ) ), 
                  stripslashes_deep( $gifthunt_data->success_email_body )
                );

      // send the final mail to hunter
      $mail_result = wp_mail( $to, $subject, $message, "From: " . $from_name . " <" . $from_email . ">\r\nContent-Type: text/html; charset=UTF-8" );
    }

    // send back gift to frontend
    $result["status"] = "success";
    $result["data"] = stripslashes_deep( $collected_gift["gift"] );

    return $result;
  }
}